/*
 * OCSP responder
 * by Massimiliano Pala (madwolf@openca.org)
 * OpenCA project 2001
 *
 * Copyright (c) 2001 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#include "general.h"

#ifdef HAVE_ENGINE
#include <openssl/engine.h>
#endif

#include <time.h>
#include <sys/wait.h>

#include "ocspd_engine.h"

#include <sys/types.h>
#include <unistd.h>

EVP_PKEY *ocspd_load_key ( OCSPD_CONFIG * ocspd_conf, ENGINE *engine ) {

	int keyform = FORMAT_PEM;
	// char *keyfile = NULL;
	// char *section = NULL;
	BIO *keyf = NULL;

	EVP_PKEY *ret_pkey = NULL;

        /* Reading Private key file */
	/*
        if ((ocspd_conf->keyfile == NULL) && 
			((ocspd_conf->keyfile = 
				NCONF_get_string(ocspd_conf->conf, section,
					ENV_PRIVATE_KEY)) == NULL)) {
                lookup_fail(section,ENV_PRIVATE_KEY);
		goto err;
       	}
	*/
	if (ocspd_conf->verbose ) {
		syslog(LOG_INFO, "INFO::Loading Key %s", ocspd_conf->keyfile );
	}

#ifdef HAVE_ENGINE
	if( (ocspd_conf->engine_id != NULL) && (ocspd_conf->keyfile) && 
			(strncmp_nocase("engine:", ocspd_conf->keyfile, 7) 
								== 0) ) {
		if( ocspd_conf->verbose ) {
			syslog(LOG_INFO, "Private Key in HSM [ id = %s ]", 
							ocspd_conf->keyfile+7);
		};

		keyform = FORMAT_ENGINE;
	} else {
        	if( ocspd_conf->verbose )
               		syslog(LOG_INFO,"Reading Private Key file %s", 
							ocspd_conf->keyfile);
        	if ((keyf=BIO_new_file( ocspd_conf->keyfile, "r")) == NULL) {
			syslog( LOG_ERR,"cannot open BIO file, why ?" );
               		goto err;
		}
	}
#else
        if( ocspd_conf->verbose )
               	syslog(LOG_INFO,"Reading Private Key file %s", 
							ocspd_conf->keyfile);
        if ((keyf=BIO_new_file( ocspd_conf->keyfile, "r")) == NULL) {
		syslog( LOG_ERR,"cannot open BIO file, why ?" );
               	goto err;
	}
#endif

#ifdef HAVE_ENGINE
	if( keyform == FORMAT_ENGINE ) {
		PW_CB_DATA cb_data;
                                                                                
        	cb_data.password = ocspd_conf->pwd;
        	cb_data.prompt_info = ocspd_conf->keyfile + 7;

                if( engine == NULL ) {
			syslog( LOG_ERR, "ERROR, No engine loaded!");
                        goto err;
                }
                ret_pkey = 
			ENGINE_load_private_key( engine, 
				(char *) ocspd_conf->keyfile+7, UI_OpenSSL(), 
					&cb_data);

	} else if ( keyform == FORMAT_PEM ) {
		ret_pkey = 
			PEM_read_bio_PrivateKey(keyf,NULL,NULL,ocspd_conf->pwd);
		// if (key) memset(key,0,strlen(key));
	} else {
		syslog(LOG_ERR, "ERROR, Bad input format for key file!");
		goto err;
        };
#else
	if ( keyform == FORMAT_PEM ) {
		ret_pkey = PEM_read_bio_PrivateKey(keyf, NULL, NULL,
							ocspd_conf->pwd);
	} else {
		syslog(LOG_ERR, "ERROR, bad input format for key file!");
		goto err;
        };
#endif

        if( keyf ) BIO_free( keyf );

        if( ret_pkey == NULL ) {
                syslog(LOG_ERR,"ERROR, can not load private key\n");
                goto err;
        };

	if ( keyform != FORMAT_ENGINE ) {
        	if (!X509_check_private_key(ocspd_conf->ocspd_cert,
						ret_pkey)) {
               		syslog(LOG_ERR, "ERROR, Cert and privateKey do not "
							"match!");
               		goto err;
		}
	}

	return ret_pkey;

err:

	return NULL;
}

